#========================================================================================
# package      DE_EPAGES::Packlink::t::unit::GetPacklinkUrls
# base         DE_EPAGES::Core::API::Test
# state        public
# description  Unit test for TLE/GetPacklinkUrls.t
#========================================================================================
package DE_EPAGES::Packlink::t::unit::GetPacklinkUrls;
use base DE_EPAGES::Core::API::Test;

use Test::More;
use Test::MockObject::Extends;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Packlink::API::TLE::GetPacklinkUrls;
use DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink;
use strict;

#----------------------------------------------------------------------------------------
sub test {
    my $self = shift;
    $self->_test_get_packlink_backoffice_url_positive();
    $self->_test_get_packlink_backoffice_url_without_shipping_fulfillment_types();
    $self->_test_get_packlink_backoffice_url_without_shipping_provider_methods();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub _test_get_packlink_backoffice_url_positive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Site = $self->_createSite(0);
    my $Shop = $self->_createShop(0);
    my $aParams = ['12345', 'de_DE'];

    my $MockedGetPacklinkUrls = $self->_getMockedGetPacklinkUrls($Site, $Shop);
    my $PacklinkBackOfficeUrl = $MockedGetPacklinkUrls->GET_PACKLINK_BACKOFFICE_URL('', $aParams);

    my $ExpectedPacklinkBackOfficeUrl = 'https://pro.packlink.de/epages';
    is($PacklinkBackOfficeUrl, $ExpectedPacklinkBackOfficeUrl, "$SubTestName - Tests if the Packlink back office url is correct.");

    return;
}

#----------------------------------------------------------------------------------------
sub _test_get_packlink_backoffice_url_without_shipping_fulfillment_types {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Site = $self->_createSite(1);
    my $Shop = $self->_createShop(0);
    my $aParams = ['12345'];

    my $MockedGetPacklinkUrls = $self->_getMockedGetPacklinkUrls($Site, $Shop);
    my $PacklinkBackOfficeUrl = $MockedGetPacklinkUrls->GET_PACKLINK_BACKOFFICE_URL('', $aParams);

    is($PacklinkBackOfficeUrl, undef, "$SubTestName - Tests if Packlink's back office url is empty when there are no shipping fulfillment types.");

    return;
}

#----------------------------------------------------------------------------------------
sub _test_get_packlink_backoffice_url_without_shipping_provider_methods {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Site = $self->_createSite(0);
    my $Shop = $self->_createShop(1);
    my $aParams = ['12345'];

    my $MockedGetPacklinkUrls = $self->_getMockedGetPacklinkUrls($Site, $Shop);
    my $PacklinkBackOfficeUrl = $MockedGetPacklinkUrls->GET_PACKLINK_BACKOFFICE_URL('', $aParams);

    is($PacklinkBackOfficeUrl, undef, "$SubTestName - Tests if Packlink's back office url is empty when there are no shipping provider methods.");

    return;
}

#WRAPPERS
#----------------------------------------------------------------------------------------
sub _getMockedGetPacklinkUrls {
    my $self = shift;
    my ($Site, $Shop) = @_;

    my $GetPacklinkUrls = DE_EPAGES::Packlink::API::TLE::GetPacklinkUrls->new();
    my $GetPacklinkUrlsMocked = Test::MockObject::Extends->new($GetPacklinkUrls);

    $GetPacklinkUrlsMocked->{'System'} = $Site;

    $GetPacklinkUrlsMocked->mock('_LoadRootObject', sub {
        shift;
        return $GetPacklinkUrlsMocked->{'System'};
    });

    $GetPacklinkUrlsMocked->mock('_LoadObject', sub {
        shift;
        return $Shop;
    });

    $GetPacklinkUrlsMocked->mock('_TestObject', sub {
        shift;
        return;
    });

    return $GetPacklinkUrlsMocked;
}

#----------------------------------------------------------------------------------------
sub _createSite {
    my $self = shift;
    my ($WithoutShippingFulfillmentTypes) = @_;

    my $ShippingFulfillmentTypes = $self->_getShippingFulfillmentTypes($WithoutShippingFulfillmentTypes);

    my $Attributes = {
        'ShippingFulfillmentTypes'  => $ShippingFulfillmentTypes,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site',$Attributes);
}

#----------------------------------------------------------------------------------------
sub _getShippingFulfillmentTypes {
    my $self = shift;
    my ($WithoutShippingFulfillmentTypes) = @_;

    if ($WithoutShippingFulfillmentTypes) {
        return [];
    }

    my $Attributes = {
        'Alias'                 => 'Packlink',
        'AdminURL'              => 'https://gopacklink.com/epages/orders/',
        'AdminLocalizedUrls'    => 'https://pro.packlink.de/epages'
    };
    return [DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType', $Attributes)];
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my ($WithoutShippingProviderMethods) = @_;

    my $Attributes = {
        'ID'                        => '12345',
        'ShippingProviderMethods'   => $self->_getShippingProviderMethods($WithoutShippingProviderMethods),
        'Languages'                 => [{'Code2' => 'en'}, {'Code2' => 'de'}]
    };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $Attributes);

    $Shop->set({
        'Site' => $Shop,
    });

    return $Shop;
}

#----------------------------------------------------------------------------------------
sub _getShippingProviderMethods {
    my $self = shift;
    my ($WithoutShippingProviderMethods) = @_;

    if ($WithoutShippingProviderMethods) {
        return [];
    }

    my $Attributes = {
        'Alias'     => 'Packlink',
        'PacklinkID'  => '67890',
    };
    my $ShippingProviderMethodPacklink =
        DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink', $Attributes);
    my $ShippingProviderMethodPacklinkMocked = Test::MockObject::Extends->new($ShippingProviderMethodPacklink);

    $ShippingProviderMethodPacklinkMocked->mock('isUsable', sub {
        shift;
        return 1;
    });

    return [$ShippingProviderMethodPacklinkMocked];
}


__PACKAGE__->runScript();